@@ -87,6 +87,14 @@ class Url(models.Model):
         else:
             return 'invalid'
 
+    @property
+    def has_anchor(self):
+        return '#' in self.url
+
+    @property
+    def anchor(self):
+        return self.url.split('#')[1] if self.has_anchor else None
+
     @property
     def get_message(self):
         if self.last_checked:
@@ -151,6 +159,27 @@ class Url(models.Model):
         logger.debug('Internal URL: %s', prepared_url)
         return prepared_url
 
+    @cached_property
+    def external_url(self):
+        """
+        Prepare an external URL to be checked with requests:
+        - Remove hash anchors
+        - Ensure correct encoding
+        """
+        # If the URL is internal, return `None`
+        if self.internal:
+            return None
+
+        # Remove URL fragment identifiers
+        external_url = self.url.rsplit('#')[0]
+        # Check that non-ascii chars are properly encoded
+        try:
+            external_url.encode('ascii')
+        except UnicodeEncodeError:
+            external_url = iri_to_uri(external_url)
+        logger.debug('External URL: %s', external_url)
+        return external_url
+
     @property
     def internal(self):
         """
@@ -176,40 +205,47 @@ class Url(models.Model):
         self.status = False
 
         if check_internal and self.internal:
-            self._check_internal(self.internal_url)
+            self.check_internal()
 
         elif check_external and self.external:
-            self._check_external(self.url, external_recheck_interval)
+            self.check_external(external_recheck_interval)
 
         else:
             return None
 
         return self.status
 
-    def _check_internal(self, tested_url):
-        logger.debug('checking internal link: %s', tested_url)
+    def check_internal(self):
+        """
+        Check an internal URL
+        """
+        if not self.internal:
+            logger.info('URL %r is not internal', self)
+            return None
+
+        logger.debug('checking internal link: %s', self.internal_url)
 
         from linkcheck.utils import LinkCheckHandler
 
-        if not tested_url:
+        if self.type == 'empty':
             self.message = 'Empty link'
 
-        elif tested_url.startswith('mailto:'):
+        elif self.type == 'mailto':
             self.status = None
             self.message = 'Email link (not automatically checked)'
 
-        elif tested_url.startswith('tel:'):
+        elif self.type == 'phone':
             self.status = None
             self.message = 'Phone number (not automatically checked)'
 
-        elif tested_url.startswith('#'):
+        elif self.type == 'anchor':
             self.status = None
             self.message = 'Link to within the same page (not automatically checked)'
 
-        elif tested_url.startswith(MEDIA_PREFIX):
+        elif self.type == 'file':
             # TODO: Assumes a direct mapping from media url to local filesystem path.
             # This will break quite easily for alternate setups
-            path = settings.MEDIA_ROOT + unquote(tested_url)[len(MEDIA_PREFIX) - 1:]
+            path = settings.MEDIA_ROOT + unquote(self.internal_url)[len(MEDIA_PREFIX) - 1:]
             decoded_path = html_decode(path)
             if os.path.exists(path) or os.path.exists(decoded_path):
                 self.message = 'Working file link'
@@ -231,20 +267,20 @@ class Url(models.Model):
                     html_content += getattr(instance, field, '')
                 self._check_anchor(hash, html_content)
 
-        elif tested_url.startswith('/'):
+        elif self.type == 'internal':
             old_prepend_setting = settings.PREPEND_WWW
             settings.PREPEND_WWW = False
             c = Client()
             c.handler = LinkCheckHandler()
             with modify_settings(ALLOWED_HOSTS={'append': 'testserver'}):
-                response = c.get(tested_url)
+                response = c.get(self.internal_url)
             if response.status_code == 200:
                 self.message = 'Working internal link'
                 self.status = True
             elif response.status_code == 302 or response.status_code == 301:
                 redirect_type = "permanent" if response.status_code == 301 else "temporary"
                 with modify_settings(ALLOWED_HOSTS={'append': 'testserver'}):
-                    response = c.get(tested_url, follow=True)
+                    response = c.get(self.internal_url, follow=True)
                 if response.status_code == 200:
                     self.message = f'Working {redirect_type} redirect'
                     self.status = True
@@ -252,13 +288,10 @@ class Url(models.Model):
                     self.message = f'Broken {redirect_type} redirect'
             else:
                 self.message = 'Broken internal link'
-            # see if the internal link points an anchor
-            if tested_url[-1] == '#':
-                # special case, point to #
-                self.message += ', working internal hash anchor'
-            elif tested_url.count('#'):
-                anchor = tested_url.split('#')[1]
-                self._check_anchor(anchor, response.content)
+
+            # Check the anchor (if it exists)
+            self.check_anchor(response.content)
+
             settings.PREPEND_WWW = old_prepend_setting
         else:
             self.message = 'Invalid URL'
@@ -270,8 +303,15 @@ class Url(models.Model):
         self.last_checked = now()
         self.save()
 
-    def _check_external(self, tested_url, external_recheck_interval):
-        logger.info('checking external link: %s', tested_url)
+    def check_external(self, external_recheck_interval=EXTERNAL_RECHECK_INTERVAL):
+        """
+        Check an external URL
+        """
+        if not self.external:
+            logger.info('URL %r is not external', self)
+            return None
+
+        logger.info('checking external link: %s', self.url)
         external_recheck_datetime = now() - timedelta(minutes=external_recheck_interval)
 
         if self.last_checked and (self.last_checked > external_recheck_datetime):
@@ -281,30 +321,37 @@ class Url(models.Model):
             )
             return self.status
 
-        # Remove URL fragment identifiers
-        url = tested_url.rsplit('#')[0]
-        # Check that non-ascii chars are properly encoded
-        try:
-            url.encode('ascii')
-        except UnicodeEncodeError:
-            url = iri_to_uri(url)
-
         request_params = {
             'allow_redirects': True,
             'headers': {'User-Agent': f"http://{settings.SITE_DOMAIN} Linkchecker"},
             'timeout': LINKCHECK_CONNECTION_ATTEMPT_TIMEOUT,
+            'verify': True,
         }
         try:
-            if tested_url.count('#'):
-                # We have to get the content so we can check the anchors
-                response = requests.get(url, **request_params)
-            else:
-                # Might as well just do a HEAD request
-                response = requests.head(url, **request_params)
-                # If HEAD is not allowed, let's try with GET
-                if response.status_code >= 400:
-                    logger.debug('HEAD is not allowed, retry with GET')
-                    response = requests.get(url, **request_params)
+            try:
+                # At first try a HEAD request
+                response = requests.head(self.external_url, **request_params)
+            except ConnectionError as e:
+                # This error could also be caused by an incomplete root certificate bundle,
+                # so let's retry without verifying the certificate
+                if "unable to get local issuer certificate" in str(e):
+                    request_params['verify'] = False
+                    response = requests.head(self.external_url, **request_params)
+                else:
+                    # Re-raise exception if it's definitely not a false positive
+                    raise
+            # If HEAD is not allowed, let's try with GET
+            if response.status_code >= 400:
+                logger.debug('HEAD is not allowed, retry with GET')
+                response = requests.get(self.external_url, **request_params)
+            # If URL contains hash anchor and is a valid HTML document, let's repeat with GET
+            elif (
+                self.has_anchor and
+                response.ok and
+                'text/html' in response.headers.get('content-type')
+            ):
+                logger.debug('Retrieve content for anchor check')
+                response = requests.get(self.external_url, **request_params)
         except ReadTimeout:
             self.message = 'Other Error: The read operation timed out'
         except ConnectionError as e:
@@ -322,30 +369,45 @@ class Url(models.Model):
                     logger.debug('Redirect history: %r', response.history)
                     self.message = f"{response.history[0].status_code} {response.history[0].reason}"
                     self.redirect_to = response.url
-
-            if tested_url.count('#'):
-                anchor = tested_url.split('#')[1]
-                self._check_anchor(anchor, response.text, internal=False)
+            # Check the anchor (if it exists)
+            if response.request.method == 'GET':
+                self.check_anchor(response.text)
+            if not request_params['verify']:
+                self.message += ', SSL certificate could not be verified'
 
         self.last_checked = now()
         self.save()
 
-    def _check_anchor(self, anchor, html, internal=True):
+    def check_anchor(self, html):
         from linkcheck import parse_anchors
-        scope = "internal" if internal else "external"
-        try:
-            names = parse_anchors(html)
-            if anchor in names:
+
+        scope = "internal" if self.internal else "external"
+
+        # Only check when the URL contains an anchor
+        if self.has_anchor:
+            # Empty fragment '#' is always valid
+            if not self.anchor:
                 self.message += f', working {scope} hash anchor'
             else:
-                self.message += f', broken {scope} hash anchor'
-                if not TOLERATE_BROKEN_ANCHOR:
-                    self.status = False
-        except UnicodeDecodeError as e:
-            logger.debug('UnicodeDecodeError while parsing anchors: %s', e)
-            self.message += ', failed to parse HTML for anchor'
-            if not TOLERATE_BROKEN_ANCHOR:
-                self.status = False
+                try:
+                    names = parse_anchors(html)
+                # Known possible errors include: AssertionError, NotImplementedError, UnicodeDecodeError
+                except Exception as e:
+                    logger.debug(
+                        '%s while parsing anchors: %s',
+                        type(e).__name__,
+                        e
+                    )
+                    self.message += ', failed to parse HTML for anchor'
+                    if not TOLERATE_BROKEN_ANCHOR:
+                        self.status = False
+                else:
+                    if self.anchor in names:
+                        self.message += f', working {scope} hash anchor'
+                    else:
+                        self.message += f', broken {scope} hash anchor'
+                        if not TOLERATE_BROKEN_ANCHOR:
+                            self.status = False
 
 
 class Link(models.Model):
