@@ -174,6 +174,27 @@ class ExternalCheckTestCase(LiveServerTestCase):
         self.assertEqual(uv.message, 'SSL Error: wrong version number')
         self.assertEqual(uv.redirect_to, '')
 
+    def test_external_check_200_incomplete_cert(self):
+        uv = Url(url="https://incomplete-chain.badssl.com/")
+        uv.check_url()
+        self.assertEqual(uv.status, True)
+        self.assertEqual(uv.message, '200 OK, SSL certificate could not be verified')
+        self.assertEqual(uv.redirect_to, '')
+
+    def test_external_check_200_broken_anchor_incomplete_cert(self):
+        uv = Url(url="https://incomplete-chain.badssl.com/#broken")
+        uv.check_url()
+        self.assertEqual(uv.status, True)
+        self.assertEqual(uv.message, '200 OK, broken external hash anchor, SSL certificate could not be verified')
+        self.assertEqual(uv.redirect_to, '')
+
+    def test_external_check_404_incomplete_cert(self):
+        uv = Url(url="https://incomplete-chain.badssl.com/404")
+        uv.check_url()
+        self.assertEqual(uv.status, False)
+        self.assertEqual(uv.message, '404 Not Found, SSL certificate could not be verified')
+        self.assertEqual(uv.redirect_to, '')
+
     def test_external_check_200_utf8(self):
         uv = Url(url=f"{self.live_server_url}/http/200/r%C3%BCckmeldung/")
         uv.check_url()
@@ -274,6 +295,18 @@ class ExternalCheckTestCase(LiveServerTestCase):
         self.assertEqual(uv.status, True)
         self.assertEqual(uv.message, "302 Found, broken external hash anchor")
 
+    def test_video_with_time_anchor(self):
+        uv = Url(url=f"{self.live_server_url}/static-files/video.mp4#t=2.0")
+        uv.check_url()
+        self.assertEqual(uv.status, True)
+        self.assertEqual(uv.message, "200 OK")
+
+    def test_forged_video_with_time_anchor(self):
+        uv = Url(url=f"{self.live_server_url}/static-files/fake-video.mp4#t=2.0")
+        uv.check_url()
+        self.assertEqual(uv.status, True)
+        self.assertEqual(uv.message, "200 OK, failed to parse HTML for anchor")
+
 
 class ModelTestCase(TestCase):
 
@@ -518,7 +551,7 @@ class QueueTests(TestCase):
 
 class ViewTestCase(TestCase):
     def setUp(self):
-        User.objects.create_superuser('admin', 'admin@example.org', 'password')
+        self.user = User.objects.create_superuser('admin', 'admin@example.org', 'password')
 
     def test_display_url(self):
         Book.objects.create(
@@ -532,12 +565,46 @@ class ViewTestCase(TestCase):
         )
 
     def test_report_view(self):
-        self.client.login(username='admin', password='password')
+        self.client.force_login(self.user)
         response = self.client.get(reverse('linkcheck_report'))
         self.assertContains(response, "<h1>Link Checker</h1>")
 
+    def test_report_ignore_unignore(self):
+        Author.objects.create(name="John Smith", website="http://www.example.org/john")
+        self.client.force_login(self.user)
+        link = Link.objects.first()
+        self.assertFalse(link.ignore)
+        response = self.client.post(
+            reverse('linkcheck_report') + f"?ignore={link.pk}",
+            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
+        )
+        self.assertEqual(response.json(), {'link': link.pk})
+        link.refresh_from_db()
+        self.assertTrue(link.ignore)
+        response = self.client.post(
+            reverse('linkcheck_report') + f"?unignore={link.pk}",
+            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
+        )
+        self.assertEqual(response.json(), {'link': link.pk})
+        link.refresh_from_db()
+        self.assertFalse(link.ignore)
+
+    def test_report_recheck(self):
+        Author.objects.create(name="John Smith", website="http://www.example.org/john")
+        self.client.force_login(self.user)
+        link = Link.objects.first()
+        response = self.client.post(
+            reverse('linkcheck_report') + f"?recheck={link.pk}",
+            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
+        )
+        self.assertEqual(response.json(), {
+            'colour': 'red',
+            'links': [link.pk],
+            'message': '404 Not Found',
+        })
+
     def test_coverage_view(self):
-        self.client.login(username='admin', password='password')
+        self.client.force_login(self.user)
         response = self.client.get(reverse('linkcheck_coverage'))
         self.assertContains(
             response,
